#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_xml
short_description: Manages XML file content on Windows hosts
description:
    - Manages XML nodes, attributes and text, using xpath to select which xml nodes need to be managed.
    - XML fragments, formatted as strings, are used to specify the desired state of a part or parts of XML files on remote Windows servers.
    - For non-Windows targets, use the M(community.general.xml) module instead.
options:
    attribute:
        description:
        - The attribute name if the type is 'attribute'.
        - Required if C(type=attribute).
        type: str
    count:
        description:
        - When set to C(yes), return the number of nodes matched by I(xpath).
        type: bool
        default: false
    content:
        description:
        - Returns the content of the nodes matched by I(xpath).
        type: str
        choices: [ attribute, text ]
        version_added: '3.2.0'
    backup:
        description:
        - Determine whether a backup should be created.
        - When set to C(yes), create a backup file including the timestamp information
          so you can get the original file back if you somehow clobbered it incorrectly.
        type: bool
        default: no
    fragment:
        description:
        - The string representation of the XML fragment expected at xpath.  Since ansible 2.9 not required when I(state=absent) or when I(count=yes).
        - Not required when I(content) is set.
        type: str
        required: false
        aliases: [ xmlstring ]
    path:
        description:
        - Path to the file to operate on.
        type: path
        required: true
        aliases: [ dest, file ]
    state:
        description:
        - Set or remove the nodes (or attributes) matched by I(xpath).
        type: str
        default: present
        choices: [ present, absent ]
    type:
        description:
        - The type of XML node you are working with.
        type: str
        default: element
        choices: [ attribute, element, text ]
    xpath:
        description:
        - Xpath to select the node or nodes to operate on.
        type: str
        required: true
    preserve_whitespace:
        description:
        - Whether to preserve whitespace in the XML.
        type: bool
        default: false
        version_added: "3.2.0"
author:
    - Richard Levenberg (@richardcs)
    - Jon Hawkesworth (@jhawkesworth)
notes:
    - Only supports operating on xml elements, attributes and text.
    - Namespace, processing-instruction, command and document node types cannot be modified with this module.
seealso:
    - module: community.general.xml
      description: XML manipulation for Posix hosts.
    - name: w3shools XPath tutorial
      description: A useful tutorial on XPath
      link: https://www.w3schools.com/xml/xpath_intro.asp
'''

EXAMPLES = r'''
- name: Apply our filter to Tomcat web.xml
  community.windows.win_xml:
    path: C:\apache-tomcat\webapps\myapp\WEB-INF\web.xml
    fragment: '<filter><filter-name>MyFilter</filter-name><filter-class>com.example.MyFilter</filter-class></filter>'
    xpath: '/*'

- name: Apply sslEnabledProtocols to Tomcat's server.xml
  community.windows.win_xml:
    path: C:\Tomcat\conf\server.xml
    xpath: '//Server/Service[@name="Catalina"]/Connector[@port="9443"]'
    attribute: 'sslEnabledProtocols'
    fragment: 'TLSv1,TLSv1.1,TLSv1.2'
    type: attribute

- name: remove debug configuration nodes from nlog.conf
  community.windows.win_xml:
    path: C:\IISApplication\nlog.conf
    xpath: /nlog/rules/logger[@name="debug"]/descendant::*
    state: absent

- name: count configured connectors in Tomcat's server.xml
  community.windows.win_xml:
    path: C:\Tomcat\conf\server.xml
    xpath: //Server/Service/Connector
    count: true
  register: connector_count

- name: show connector count
  ansible.builtin.debug:
    msg: Connector count is {{ connector_count.count }}

- name: ensure all lang=en attributes to lang=nl
  community.windows.win_xml:
    path: C:\Data\Books.xml
    xpath: //@[lang="en"]
    attribute: lang
    fragment: nl
    type: attribute

- name: return attributes of node works node
  community.windows.win_xml:
    path: C:\Data\Books.xml
    xpath: /books/works
    content: attribute
  register: works_attributes

- name: show /books/works attribute
  ansible.builtin.debug:
    var: works_attributes
'''

RETURN = r'''
backup_file:
    description: Name of the backup file that was created.
    returned: if backup=yes
    type: str
    sample: C:\Path\To\File.txt.11540.20150212-220915.bak
count:
    description: Number of nodes matched by xpath.
    returned: if count=yes
    type: int
    sample: 33
matches:
    description: Nodes that were read with content
    returned: if content=attribute or content=text
    type: list
    sample: nodes attributes
msg:
    description: What was done.
    returned: always
    type: str
    sample: "xml added"
err:
    description: XML comparison exceptions.
    returned: always, for type element and -vvv or more
    type: list
    sample: attribute mismatch for actual=string
'''



#!/usr/bin/python
#
# Copyright (c) 2018, Bojan Vitnik <bvitnik@mainstream.rs>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: xenserver_guest_info
short_description: Gathers information for virtual machines running on Citrix Hypervisor/XenServer host or pool
description: This module can be used to gather essential VM facts.
author:
  - Bojan Vitnik (@bvitnik) <bvitnik@mainstream.rs>
notes:
  - Minimal supported version of XenServer is 5.6.
  - Module was tested with XenServer 6.5, 7.1, 7.2, 7.6, Citrix Hypervisor 8.0, XCP-ng 7.6 and 8.0.
  - 'To acquire XenAPI Python library, just run C(pip install XenAPI) on your Ansible Control Node. The library can also be
    found inside Citrix Hypervisor/XenServer SDK (downloadable from Citrix website). Copy the C(XenAPI.py) file from the SDK
    to your Python site-packages on your Ansible Control Node to use it. Latest version of the library can also be acquired
    from GitHub: U(https://raw.githubusercontent.com/xapi-project/xen-api/master/scripts/examples/python/XenAPI/XenAPI.py).'
  - If no scheme is specified in C(hostname), module defaults to C(http://) because C(https://) is problematic in most setups.
    Make sure you are accessing XenServer host in trusted environment or use C(https://) scheme explicitly.
  - To use C(https://) scheme for C(hostname) you have to either import host certificate to your OS certificate store or use
    O(validate_certs=no) which requires XenAPI library from XenServer 7.2 SDK or newer and Python 2.7.9 or newer.
requirements:
  - XenAPI
options:
  name:
    description:
      - Name of the VM to gather facts from.
      - VMs running on XenServer do not necessarily have unique names. The module fails if multiple VMs with same name are
        found.
      - In case of multiple VMs with same name, use O(uuid) to uniquely specify VM to manage.
      - This parameter is case sensitive.
    type: str
    aliases: [name_label]
  uuid:
    description:
      - UUID of the VM to gather fact of. This is XenServer's unique identifier.
      - It is required if name is not unique.
    type: str
extends_documentation_fragment:
  - community.general._xenserver.documentation
  - community.general._attributes
  - community.general._attributes.info_module
"""

EXAMPLES = r"""
- name: Gather facts
  community.general.xenserver_guest_info:
    hostname: "{{ xenserver_hostname }}"
    username: "{{ xenserver_username }}"
    password: "{{ xenserver_password }}"
    name: testvm_11
  delegate_to: localhost
  register: facts
"""

RETURN = r"""
instance:
  description: Metadata about the VM.
  returned: always
  type: dict
  sample:
    {
      "cdrom": {
        "type": "none"
      },
      "customization_agent": "native",
      "disks": [
        {
          "name": "testvm_11-0",
          "name_desc": "",
          "os_device": "xvda",
          "size": 42949672960,
          "sr": "Local storage",
          "sr_uuid": "0af1245e-bdb0-ba33-1446-57a962ec4075",
          "uuid": "3f98b388-b2c0-4355-9a01-15c0e61b5a76",
          "vbd_userdevice": "0",
          "vdi_type": "vhd"
        },
        {
          "name": "testvm_11-1",
          "name_desc": "",
          "os_device": "xvdb",
          "size": 42949672960,
          "sr": "Local storage",
          "sr_uuid": "0af1245e-bdb0-ba33-1446-57a962ec4075",
          "uuid": "a1b2c3d4-e5f6-7890-abcd-ef1234567890",
          "vbd_userdevice": "1",
          "vdi_type": "vhd"
        }
      ],
      "domid": "56",
      "folder": "",
      "hardware": {
        "memory_mb": 8192,
        "num_cpu_cores_per_socket": 2,
        "num_cpus": 4
      },
      "home_server": "",
      "is_template": false,
      "name": "testvm_11",
      "name_desc": "",
      "networks": [
        {
          "gateway": "192.168.0.254",
          "gateway6": "fc00::fffe",
          "ip": "192.168.0.200",
          "ip6": [
            "fe80:0000:0000:0000:e9cb:625a:32c5:c291",
            "fc00:0000:0000:0000:0000:0000:0000:0001"
          ],
          "mac": "ba:91:3a:48:20:76",
          "mtu": "1500",
          "name": "Pool-wide network associated with eth1",
          "netmask": "255.255.255.128",
          "prefix": "25",
          "prefix6": "64",
          "vif_device": "0"
        }
      ],
      "other_config": {
        "base_template_name": "Windows Server 2016 (64-bit)",
        "import_task": "OpaqueRef:e43eb71c-45d6-5351-09ff-96e4fb7d0fa5",
        "install-methods": "cdrom",
        "instant": "true",
        "mac_seed": "f83e8d8a-cfdc-b105-b054-ef5cb416b77e"
      },
      "platform": {
        "acpi": "1",
        "apic": "true",
        "cores-per-socket": "2",
        "device_id": "0002",
        "hpet": "true",
        "nx": "true",
        "pae": "true",
        "timeoffset": "-25200",
        "vga": "std",
        "videoram": "8",
        "viridian": "true",
        "viridian_reference_tsc": "true",
        "viridian_time_ref_count": "true"
      },
      "state": "poweredon",
      "uuid": "e3c0b2d5-5f05-424e-479c-d3df8b3e7cda",
      "xenstore_data": {
        "vm-data": ""
      }
    }
"""


from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._xenserver import (
    XenServerObject,
    gather_vm_facts,
    gather_vm_params,
    get_object_ref,
    xenserver_common_argument_spec,
)


class XenServerVM(XenServerObject):
    """Class for managing XenServer VM.

    Attributes:
        vm_ref (str): XAPI reference to VM.
        vm_params (dict): A dictionary with VM parameters as returned
            by gather_vm_params() function.
    """

    def __init__(self, module):
        """Inits XenServerVM using module parameters.

        Args:
            module: Reference to AnsibleModule object.
        """
        super().__init__(module)

        self.vm_ref = get_object_ref(
            self.module,
            self.module.params["name"],
            self.module.params["uuid"],
            obj_type="VM",
            fail=True,
            msg_prefix="VM search: ",
        )
        self.gather_params()

    def gather_params(self):
        """Gathers all VM parameters available in XAPI database."""
        self.vm_params = gather_vm_params(self.module, self.vm_ref)

    def gather_facts(self):
        """Gathers and returns VM facts."""
        return gather_vm_facts(self.module, self.vm_params)


def main():
    argument_spec = xenserver_common_argument_spec()
    argument_spec.update(
        name=dict(type="str", aliases=["name_label"]),
        uuid=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_one_of=[
            ["name", "uuid"],
        ],
    )

    result = {"failed": False, "changed": False}

    # Module will exit with an error message if no VM is found.
    vm = XenServerVM(module)

    # Gather facts.
    result["instance"] = vm.gather_facts()

    if result["failed"]:
        module.fail_json(**result)
    else:
        module.exit_json(**result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
#
# Copyright: (c) 2021, F5 Networks Inc.
# GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# This collection version needs to be updated at each release
CURRENT_COLL_VERSION = "1.42.0"



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2015, Phil Schwartz <schwartzmx@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_unzip
short_description: Unzips compressed files and archives on the Windows node
description:
- Unzips compressed files and archives.
- Supports .zip files natively via the .NET BCL.
- Supports tar-based formats (C(.tar), C(.tar.gz)/C(.tgz), C(.tar.bz2)/C(.tbz2), C(.tar.xz)/C(.txz))
  via C(tar.exe) as of C(community.windows 3.2.0). C(tar.exe) ships with Windows 10 build 17063 and
  later and Windows Server 2019 and later, with no additional software required.
- Supports other formats (C(.gz), C(.bz2), C(.msu) and others) via the PowerShell Community
  Extensions (PSCX) module. PSCX is also required when using the O(recurse) or O(password) options,
  and for tar-based formats prior to C(community.windows 3.2.0).
- For non-Windows targets, use the M(ansible.builtin.unarchive) module instead.
requirements:
- PSCX for formats other than C(.zip) and tar-based, or when using O(recurse) or O(password).
  Also required for tar-based formats prior to C(community.windows 3.2.0), or on older OS versions
  which did not ship C(tar.exe).
options:
  src:
    description:
      - File to be unzipped (provide absolute path).
    type: path
    required: yes
  dest:
    description:
      - Destination of zip file (provide absolute path of directory). If it does not exist, the directory will be created.
    type: path
    required: yes
  delete_archive:
    description:
      - Remove the zip file, after unzipping.
    type: bool
    default: no
    aliases: [ rm ]
  recurse:
    description:
      - Recursively expand zipped files within the src file.
      - Setting to a value of C(yes) requires the PSCX module to be installed.
    type: bool
    default: no
  creates:
    description:
      - If this file or directory exists the specified src will not be extracted.
    type: path
  password:
    description:
      - If a zip file is encrypted with password.
      - Passing a value to a password parameter requires the PSCX module to be installed.
notes:
- This module is not really idempotent, it will extract the archive every time, and report a change.
- For tar-based formats (C(.tar), C(.tar.gz), C(.tgz), C(.tar.bz2), C(.tbz2), C(.tar.xz), C(.txz))
  the module uses C(%SystemRoot%\System32\tar.exe) when available, requiring no extra software.
  On older systems where C(tar.exe) is not present, the module falls back to PSCX.
- For all other compression types, or when O(recurse) or O(password) are used, the
  PowerShellCommunityExtensions (PSCX) module is required. If the destination directory does not
  exist, it will be created before unzipping the file. Specifying rm parameter will force removal
  of the src file after extraction.
seealso:
- module: ansible.builtin.unarchive
author:
- Phil Schwartz (@schwartzmx)
'''

EXAMPLES = r'''
# This unzips a library that was downloaded with win_get_url, and removes the file after extraction
# $ ansible -i hosts -m win_unzip -a "src=C:\LibraryToUnzip.zip dest=C:\Lib remove=yes" all

- name: Unzip a bz2 (BZip) file
  community.windows.win_unzip:
    src: C:\Users\Phil\Logs.bz2
    dest: C:\Users\Phil\OldLogs
    creates: C:\Users\Phil\OldLogs

- name: Unzip gz log
  community.windows.win_unzip:
    src: C:\Logs\application-error-logs.gz
    dest: C:\ExtractedLogs\application-error-logs

- name: Extract a tar.gz archive (uses tar.exe on Windows 10/Server 2019+, no extra software needed)
  community.windows.win_unzip:
    src: C:\Downloads\package.tar.gz
    dest: C:\Extracted\package
    delete_archive: true

# Unzip .zip file, recursively decompresses the contained .gz files and removes all unneeded compressed files after completion.
- name: Recursively decompress GZ files in ApplicationLogs.zip
  community.windows.win_unzip:
    src: C:\Downloads\ApplicationLogs.zip
    dest: C:\Application\Logs
    recurse: true
    delete_archive: true

# PSCX is required for: non-tar formats other than .zip, recurse, and password-protected archives.
- name: Install PSCX
  community.windows.win_psmodule:
    name: Pscx
    state: present

- name: Unzip .7z file which is password encrypted
  community.windows.win_unzip:
    src: C:\Downloads\ApplicationLogs.7z
    dest: C:\Application\Logs
    password: abcd
    delete_archive: true
'''

RETURN = r'''
dest:
    description: The provided destination path
    returned: always
    type: str
    sample: C:\ExtractedLogs\application-error-logs
removed:
    description: Whether the module did remove any files during task run
    returned: always
    type: bool
    sample: true
src:
    description: The provided source path
    returned: always
    type: str
    sample: C:\Logs\application-error-logs.gz
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_macsec_policy
short_description: MACsec Policy configuration for Cisco Intersight
description:
  - Manages MACsec Policy configuration on Cisco Intersight.
  - A policy to configure MACsec encryption settings for fabric interconnect ports.
  - Supports primary keychain and optional fallback keychain configuration.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles, Policies, and Pools that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the MACsec Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the MACsec Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  cipher_suite:
    description:
      - Cipher suite to be used for MACsec encryption.
    type: str
    choices: ['gcm-aes-xpn-256', 'gcm-aes-128', 'gcm-aes-256', 'gcm-aes-xpn-128']
    default: 'gcm-aes-xpn-256'
  confidentiality_offset:
    description:
      - The MACsec confidentiality offset specifies the number of bytes starting from the frame header.
      - MACsec encrypts only the bytes after the offset in a frame.
    type: str
    choices: ['conf-offset-0', 'conf-offset-30', 'conf-offset-50']
    default: 'conf-offset-0'
  security_policy:
    description:
      - The security policy specifies the level of MACsec enforcement on network traffic.
      - C(should-secure) allows unencrypted traffic until MKA session is secured.
      - C(must-secure) only allows MACsec encrypted traffic.
    type: str
    choices: ['should-secure', 'must-secure']
    default: 'should-secure'
  key_server_priority:
    description:
      - Key server is selected by comparing priority values during MKA message exchange.
      - Valid values range from 0 to 255.
      - Lower value means higher chance of being selected as key server.
    type: int
    default: 16
  sak_expiry_time:
    description:
      - Time in seconds to force secure association key (SAK) rekey.
      - Valid range is from 60 to 2592000 seconds.
      - When set to 0 or not configured, SAK rekey interval is determined based on interface speed.
    type: int
  replay_window_size:
    description:
      - Defines the size of the replay protection window.
      - Determines the number of packets that can be received out of order without being considered replay attacks.
      - Valid range is from 0 to 596000000.
    type: int
    default: 148809600
  include_icv_indicator:
    description:
      - Configures inclusion of the optional integrity check value (ICV) indicator.
      - Part of the transmitted MACsec key agreement protocol data unit (PDU).
    type: bool
    default: false
  eapol_mac_address:
    description:
      - MAC address to use in extensible authentication protocol over LAN (EAPoL) for MKA PDUs.
      - EAPol MAC address should not be equal to all-zero (0000.0000.0000).
    type: str
    default: '0180.C200.0003'
  eapol_ethertype:
    description:
      - Ethertype to use in EAPoL frames for MKA PDUs.
      - The range is between 0x600 - 0xffff.
    type: str
    default: '0x888e'
  primary_keychain_name:
    description:
      - Primary keychain name for managing the default set of security keys.
    type: str
  primary_keys:
    description:
      - List of security keys for the primary keychain.
    type: list
    elements: dict
    default: []
    suboptions:
      id:
        description:
          - Key ID must have an even number of hexadecimal characters (0-9, A-F).
          - Length must be between 2 and 64 characters.
        type: str
        required: true
      cryptographic_algorithm:
        description:
          - Cryptographic algorithm for the key.
        type: str
        choices: ['aes-256-cmac', 'aes-128-cmac']
        default: 'aes-256-cmac'
      secret:
        description:
          - Key secret is a shared secret used in cryptographic operations.
          - Must start with the character 'J'.
        type: str
        required: true
      key_lifetime_always_active:
        description:
          - Indicates that the key remains active indefinitely.
          - When C(true), the key is always active.
          - When C(false), C(start_time) and C(lifetime_type) must be specified.
        type: bool
        default: true
      timezone:
        description:
          - The time zone used for key lifetime configurations.
          - Only used when C(key_lifetime_always_active) is C(false).
        type: str
        choices: ['utc', 'local']
        default: 'utc'
      start_time:
        description:
          - The time of day and date when the key becomes active.
          - Format should be ISO 8601 format (e.g., 2025-11-20T09:14:00.000Z or 2025-11-20T09:14:00.000).
          - The 'Z' suffix will be automatically added if not provided.
          - Required when C(key_lifetime_always_active) is C(false).
        type: str
      lifetime_type:
        description:
          - Indicates key lifetime behavior after start time.
          - C(never) means the key remains active indefinitely after start time.
          - C(on-this-day) means the key becomes inactive at C(end_time).
        type: str
        choices: ['never', 'on-this-day']
        default: 'never'
      end_time:
        description:
          - The time of day and date when the key becomes inactive.
          - Format should be ISO 8601 format (e.g., 2025-11-21T09:20:00.000Z or 2025-11-21T09:20:00.000).
          - The 'Z' suffix will be automatically added if not provided.
          - Required when C(lifetime_type) is C(on-this-day).
        type: str
  configure_fallback_keychain:
    description:
      - Enable configuration of fallback keychain.
    type: bool
    default: false
  fallback_keychain_name:
    description:
      - Fallback keychain name.
      - Required when C(configure_fallback_keychain) is C(true).
    type: str
  fallback_keys:
    description:
      - List of security keys for the fallback keychain.
    type: list
    elements: dict
    default: []
    suboptions:
      id:
        description:
          - Key ID must have an even number of hexadecimal characters (0-9, A-F).
          - Length must be between 2 and 64 characters.
        type: str
        required: true
      cryptographic_algorithm:
        description:
          - Cryptographic algorithm for the key.
        type: str
        choices: ['aes-256-cmac', 'aes-128-cmac']
        default: 'aes-256-cmac'
      secret:
        description:
          - Key secret is a shared secret used in cryptographic operations.
          - Must start with the character 'J'.
        type: str
        required: true
      key_lifetime_always_active:
        description:
          - Indicates that the key remains active indefinitely.
          - When C(true), the key is always active.
          - When C(false), C(start_time) and C(lifetime_type) must be specified.
        type: bool
        default: true
      timezone:
        description:
          - The time zone used for key lifetime configurations.
          - Only used when C(key_lifetime_always_active) is C(false).
        type: str
        choices: ['utc', 'local']
        default: 'utc'
      start_time:
        description:
          - The time of day and date when the key becomes active.
          - Format should be ISO 8601 format (e.g., 2025-11-20T09:14:00.000Z or 2025-11-20T09:14:00.000).
          - The 'Z' suffix will be automatically added if not provided.
          - Required when C(key_lifetime_always_active) is C(false).
        type: str
      lifetime_type:
        description:
          - Indicates key lifetime behavior after start time.
          - C(never) means the key remains active indefinitely after start time.
          - C(on-this-day) means the key becomes inactive at C(end_time).
        type: str
        choices: ['never', 'on-this-day']
        default: 'never'
      end_time:
        description:
          - The time of day and date when the key becomes inactive.
          - Format should be ISO 8601 format (e.g., 2025-11-21T09:20:00.000Z or 2025-11-21T09:20:00.000).
          - The 'Z' suffix will be automatically added if not provided.
          - Required when C(lifetime_type) is C(on-this-day).
        type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create MACsec Policy with Primary Keychain Only
  cisco.intersight.intersight_macsec_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "macsec-policy-01"
    description: "MACsec policy with primary keychain"
    cipher_suite: "gcm-aes-xpn-256"
    security_policy: "should-secure"
    primary_keychain_name: "primary-keychain"
    primary_keys:
      - id: "1234"
        cryptographic_algorithm: "aes-256-cmac"
        secret: >-
          Ja1b2c3d4e5f6a7b8c9d0e1f2a3b4c5d6e7f8a9bd2e3f4a5b6c7d8e9f0a1b2c3d4e5f6a7b8c9d0e1f2a3b4c5d6e7f8a9b0c122222222222222222222222222222222222222222222
        key_lifetime_always_active: true
    tags:
      - Key: Environment
        Value: Production
    state: present

- name: Create MACsec Policy with Fallback Keychain
  cisco.intersight.intersight_macsec_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "macsec-policy-02"
    description: "MACsec policy with fallback keychain"
    cipher_suite: "gcm-aes-256"
    security_policy: "must-secure"
    key_server_priority: 32
    sak_expiry_time: 3600
    primary_keychain_name: "primary-keychain"
    primary_keys:
      - id: "ABCD"
        cryptographic_algorithm: "aes-256-cmac"
        secret: >-
          Ja1b2c3d4e5f6a7b8c9d0e1f2a3b4c5d6e7f8a9bd2e3f4a5b6c7d8e9f0a1b2c3d4e5f6a7b8c9d0e1f2a3b4c5d6e7f8a9b0c122222222222222222222222222222222222222222222
    configure_fallback_keychain: true
    fallback_keychain_name: "fallback-keychain"
    fallback_keys:
      - id: "EF12"
        cryptographic_algorithm: "aes-128-cmac"
        secret: "Jf1e2d3c4b5a6f7e8d9c0b1a2f3e4d5c6b7a8f9e0d1c2b3a4f5e6d7c8b9a0f1e2d3c4b5a6f7e8d9c0b1a2f3e4d5c6b7a8f9e0d1c2b3a4f5e6d7c8b9a0f1e2d3c4b5a6f7e8d9c0"
    state: present

- name: Create MACsec Policy with Custom Settings
  cisco.intersight.intersight_macsec_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "macsec-policy-custom"
    description: "MACsec policy with custom settings"
    cipher_suite: "gcm-aes-xpn-128"
    confidentiality_offset: "conf-offset-30"
    security_policy: "must-secure"
    key_server_priority: 64
    sak_expiry_time: 7200
    replay_window_size: 200000000
    include_icv_indicator: true
    eapol_mac_address: "0180.C200.0004"
    eapol_ethertype: "0x88e5"
    primary_keychain_name: "custom-keychain"
    primary_keys:
      - id: "2468"
        cryptographic_algorithm: "aes-256-cmac"
        secret: "Jabcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz"
    state: present

- name: Create MACsec Policy with Scheduled Key Lifetime (Never Expires)
  cisco.intersight.intersight_macsec_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "macsec-policy-scheduled-never"
    description: "MACsec policy with scheduled key that never expires"
    primary_keychain_name: "scheduled-keychain"
    primary_keys:
      - id: "3456"
        cryptographic_algorithm: "aes-256-cmac"
        secret: "Jabcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz"
        key_lifetime_always_active: false
        timezone: "local"
        start_time: "2025-11-20T10:00:00.000"
        lifetime_type: "never"
    state: present

- name: Create MACsec Policy with Scheduled Key Lifetime (With End Time)
  cisco.intersight.intersight_macsec_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "macsec-policy-scheduled-endtime"
    description: "MACsec policy with scheduled key with end time"
    primary_keychain_name: "scheduled-keychain-endtime"
    primary_keys:
      - id: "4567"
        cryptographic_algorithm: "aes-256-cmac"
        secret: "Jabcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz"
        key_lifetime_always_active: false
        timezone: "utc"
        start_time: "2025-11-20T09:20:00.000Z"
        lifetime_type: "on-this-day"
        end_time: "2025-11-21T09:20:00.000Z"
    state: present

- name: Delete MACsec Policy
  cisco.intersight.intersight_macsec_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "macsec-policy-01"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "macsec-policy-01",
        "ObjectType": "fabric.MacsecPolicy",
        "Description": "MACsec policy with primary keychain",
        "CipherSuite": "GCM-AES-XPN-256",
        "ConfidentialityOffset": "CONF-OFFSET-0",
        "SecurityPolicy": "Should-secure",
        "KeyServerPriority": 16,
        "SakExpiryTime": 0,
        "ReplayWindowSize": 148809600,
        "IncludeIcvIndicator": false,
        "MacSecEaPol": {
            "EaPolMacAddress": "0180.C200.0003",
            "EaPolEthertype": "0x888e"
        },
        "PrimaryKeyChain": {
            "Name": "primary-keychain",
            "SecKeys": [
                {
                    "Id": "1234",
                    "CryptographicAlgorithm": "AES_256_CMAC",
                    "KeyType": "Type-6",
                    "SendLifetimeUnlimited": true,
                    "SendLifetimeInfinite": false,
                    "IsOctetStringSet": true
                }
            ]
        },
        "FallbackKeyChain": {
            "Name": "",
            "SecKeys": null
        },
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def validate_key_id(key_id, field_name):
    """
    Validate key ID format.
    """
    if not key_id:
        return False, f"{field_name} is required"
    if len(key_id) < 2 or len(key_id) > 64:
        return False, f"{field_name} must be between 2 and 64 characters"
    if len(key_id) % 2 != 0:
        return False, f"{field_name} must have an even number of characters"
    if not all(c in '0123456789ABCDEFabcdef' for c in key_id):
        return False, f"{field_name} must contain only hexadecimal characters (0-9, A-F)"
    return True, None


def validate_secret(secret, field_name):
    """
    Validate secret format.
    """
    if not secret:
        return False, f"{field_name} is required"
    if not secret.startswith('J'):
        return False, f"{field_name} must start with the character 'J'"
    return True, None


def validate_macsec_configuration(module):
    """
    Validate MACsec Policy configuration parameters.
    """
    params = module.params
    if params['state'] == 'present':
        if not params.get('primary_keychain_name'):
            module.fail_json(msg="primary_keychain_name is required when state is present")

        if params.get('key_server_priority') is not None:
            if params['key_server_priority'] < 0 or params['key_server_priority'] > 255:
                module.fail_json(msg="key_server_priority must be between 0 and 255")

        if params.get('sak_expiry_time') is not None and params['sak_expiry_time'] != 0:
            if params['sak_expiry_time'] < 60 or params['sak_expiry_time'] > 2592000:
                module.fail_json(msg="sak_expiry_time must be between 60 and 2592000 seconds, or 0 for auto-determination")

        if params.get('replay_window_size') is not None:
            if params['replay_window_size'] < 0 or params['replay_window_size'] > 596000000:
                module.fail_json(msg="replay_window_size must be between 0 and 596000000")

        if params.get('eapol_mac_address'):
            if params['eapol_mac_address'].replace('.', '').lower() == '000000000000':
                module.fail_json(msg="eapol_mac_address cannot be all zeros (0000.0000.0000)")

        for key in params.get('primary_keys', []):
            is_valid, error_msg = validate_key_id(key.get('id'), 'primary_keys.id')
            if not is_valid:
                module.fail_json(msg=error_msg)
            is_valid, error_msg = validate_secret(key.get('secret'), 'primary_keys.secret')
            if not is_valid:
                module.fail_json(msg=error_msg)
                if not key.get('key_lifetime_always_active', True):
                    if not key.get('start_time'):
                        module.fail_json(msg="primary_keys.start_time is required when key_lifetime_always_active is false")
                    if key.get('lifetime_type') == 'on-this-day' and not key.get('end_time'):
                        module.fail_json(msg="primary_keys.end_time is required when lifetime_type is 'on-this-day'")

        if params.get('configure_fallback_keychain'):
            if not params.get('fallback_keychain_name'):
                module.fail_json(msg="fallback_keychain_name is required when configure_fallback_keychain is true")
            for key in params.get('fallback_keys', []):
                is_valid, error_msg = validate_key_id(key.get('id'), 'fallback_keys.id')
                if not is_valid:
                    module.fail_json(msg=error_msg)
                is_valid, error_msg = validate_secret(key.get('secret'), 'fallback_keys.secret')
                if not is_valid:
                    module.fail_json(msg=error_msg)
                if not key.get('key_lifetime_always_active', True):
                    if not key.get('start_time'):
                        module.fail_json(msg="fallback_keys.start_time is required when key_lifetime_always_active is false")
                    if key.get('lifetime_type') == 'on-this-day' and not key.get('end_time'):
                        module.fail_json(msg="fallback_keys.end_time is required when lifetime_type is 'on-this-day'")


def build_keychain_config(keychain_name, keys):
    """
    Build keychain configuration for API body.
    """
    keychain_config = {
        'Name': keychain_name
    }
    if keys:
        sec_keys = []
        for key in keys:
            sec_key = {
                'Id': key['id'],
                'CryptographicAlgorithm': convert_to_api_format(key.get('cryptographic_algorithm', 'aes-256-cmac'), 'cryptographic_algorithm'),
                'KeyType': 'Type-6',
                'OctetString': key['secret'],
                'SendLifetimeUnlimited': key.get('key_lifetime_always_active', True),
                'IsOctetStringSet': True
            }
            if not key.get('key_lifetime_always_active', True):
                timezone = key.get('timezone', 'utc')
                sec_key['SendLifetimeTimeZone'] = 'UTC' if timezone == 'utc' else 'Local'
                start_time = key['start_time']
                if not start_time.endswith('Z'):
                    start_time = start_time + 'Z'
                sec_key['SendLifetimeStartTime'] = start_time
                if key.get('lifetime_type', 'never') == 'never':
                    sec_key['SendLifetimeInfinite'] = True
                else:
                    sec_key['SendLifetimeInfinite'] = False
                    end_time = key['end_time']
                    if not end_time.endswith('Z'):
                        end_time = end_time + 'Z'
                    sec_key['SendLifetimeEndTime'] = end_time
            else:
                sec_key['SendLifetimeInfinite'] = False
            sec_keys.append(sec_key)
        keychain_config['SecKeys'] = sec_keys
    return keychain_config


def convert_to_api_format(value, value_type):
    """
    Convert lowercase user-facing values to API format.
    """
    if value_type == 'cipher_suite':
        return value.upper()
    elif value_type == 'confidentiality_offset':
        return value.upper()
    elif value_type == 'security_policy':
        parts = value.split('-')
        return f"{parts[0].capitalize()}-{parts[1]}"
    elif value_type == 'cryptographic_algorithm':
        return value.upper().replace('-', '_')
    return value


def build_api_body(intersight):
    """
    Build the API body for MACsec Policy configuration.
    """
    params = intersight.module.params
    intersight.api_body = {
        'Organization': {
            'Name': params['organization'],
        },
        'Name': params['name']
    }
    if params['state'] == 'present':
        intersight.set_tags_and_description()
        intersight.api_body['CipherSuite'] = convert_to_api_format(params['cipher_suite'], 'cipher_suite')
        intersight.api_body['ConfidentialityOffset'] = convert_to_api_format(params['confidentiality_offset'], 'confidentiality_offset')
        intersight.api_body['SecurityPolicy'] = convert_to_api_format(params['security_policy'], 'security_policy')
        intersight.api_body['KeyServerPriority'] = params['key_server_priority']
        intersight.api_body['ReplayWindowSize'] = params['replay_window_size']
        intersight.api_body['IncludeIcvIndicator'] = params['include_icv_indicator']
        if params.get('sak_expiry_time') is not None:
            intersight.api_body['SakExpiryTime'] = params['sak_expiry_time']

        intersight.api_body['MacSecEaPol'] = {
            'EaPolMacAddress': params['eapol_mac_address'],
            'EaPolEthertype': params['eapol_ethertype']
        }

        intersight.api_body['PrimaryKeyChain'] = build_keychain_config(
            params['primary_keychain_name'],
            params.get('primary_keys', [])
        )
        if params.get('configure_fallback_keychain') and params.get('fallback_keychain_name'):
            intersight.api_body['FallbackKeyChain'] = build_keychain_config(
                params['fallback_keychain_name'],
                params.get('fallback_keys', [])
            )
        else:
            intersight.api_body['FallbackKeyChain'] = {
                'Name': '',
                'SecKeys': None
            }


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        cipher_suite=dict(type='str', choices=['gcm-aes-xpn-256', 'gcm-aes-128', 'gcm-aes-256', 'gcm-aes-xpn-128'], default='gcm-aes-xpn-256'),
        confidentiality_offset=dict(type='str', choices=['conf-offset-0', 'conf-offset-30', 'conf-offset-50'], default='conf-offset-0'),
        security_policy=dict(type='str', choices=['should-secure', 'must-secure'], default='should-secure'),
        key_server_priority=dict(type='int', default=16),
        sak_expiry_time=dict(type='int'),
        replay_window_size=dict(type='int', default=148809600),
        include_icv_indicator=dict(type='bool', default=False),
        eapol_mac_address=dict(type='str', default='0180.C200.0003'),
        eapol_ethertype=dict(type='str', default='0x888e'),
        primary_keychain_name=dict(type='str'),
        primary_keys=dict(
            type='list',
            elements='dict',
            default=[],
            no_log=True,
            options=dict(
                id=dict(type='str', required=True),
                cryptographic_algorithm=dict(type='str', choices=['aes-256-cmac', 'aes-128-cmac'], default='aes-256-cmac'),
                secret=dict(type='str', required=True, no_log=True),
                key_lifetime_always_active=dict(type='bool', default=True),
                timezone=dict(type='str', choices=['utc', 'local'], default='utc'),
                start_time=dict(type='str'),
                lifetime_type=dict(type='str', choices=['never', 'on-this-day'], default='never'),
                end_time=dict(type='str')
            )
        ),
        configure_fallback_keychain=dict(type='bool', default=False),
        fallback_keychain_name=dict(type='str'),
        fallback_keys=dict(
            type='list',
            elements='dict',
            default=[],
            no_log=True,
            options=dict(
                id=dict(type='str', required=True),
                cryptographic_algorithm=dict(type='str', choices=['aes-256-cmac', 'aes-128-cmac'], default='aes-256-cmac'),
                secret=dict(type='str', required=True, no_log=True),
                key_lifetime_always_active=dict(type='bool', default=True),
                timezone=dict(type='str', choices=['utc', 'local'], default='utc'),
                start_time=dict(type='str'),
                lifetime_type=dict(type='str', choices=['never', 'on-this-day'], default='never'),
                end_time=dict(type='str')
            )
        )
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )
    if module.params['state'] == 'present':
        validate_macsec_configuration(module)

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    build_api_body(intersight)

    resource_path = '/fabric/MacSecPolicies'
    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()
